/*  $Id:$
 *
 *  xfce4-smart-plugin - a package manager applet for the xfce4 panel
 *  Copyright (c) 2007 Anders F Bjorklund <afb@users.sourceforge.net>
 *
 *  Based on: ksmarttray by Gustavo Niemeyer <niemeyer@conectiva.com>
 *            mailwatch by Brian Tarricone <bjt23@cornell.edu>,
 *                         Jasper Huijsmans <jasper@xfce.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <glib/gstdio.h>
#include <gtk/gtk.h>

#include <libxfcegui4/libxfcegui4.h>
#include <libxfce4util/libxfce4util.h>
#include <libxfce4panel/xfce-panel-plugin.h>

#include "xfcesmart.h"

/* the website url */
#define SMART_WEBSITE "http://www.labix.org/smart"

#define SMART_PATH    "/usr/local/bin:/usr/bin:/bin:/opt/gnome/bin"
#define SMART_LANG    "C" /* we're looking for certain phrases... */

#define SMART_PROGRAM "smart"
#define SMART_UPDATE  "smart-update" /* suid */

#define SMART_SUIDBIN "smart-root" /* Fedora */
#define SMART_SUDOBIN "gksudo" /* Ubuntu/etc */

#define TIMEOUT_TIME  1000 /* milliseconds */

/* prototypes */
static void
smart_construct (XfcePanelPlugin *plugin);

static void
smart_set_available(SmartPlugin *smart, gint packages);

static int
smart_update_channels( SmartPlugin *smart);

static void
smart_set_interval(SmartPlugin *smart, gint interval);

static int
smart_check_upgrades( SmartPlugin *smart);

static void
smart_sudo_smart_gui(gchar *args);


/* register the plugin */
XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (smart_construct);


static void
smart_set_icon(SmartPlugin *smart, GdkPixbuf *pix, GdkPixbuf *overlay)
{
    GdkPixbuf       *pb = gdk_pixbuf_copy(pix);
    gint w, h, ow, oh;

    w = gdk_pixbuf_get_width(pb);
    h = gdk_pixbuf_get_height(pb);
    if(overlay) {
        ow = gdk_pixbuf_get_width(overlay);
        oh = gdk_pixbuf_get_height(overlay);
        gdk_pixbuf_composite(overlay, pb, w - ow, h - oh, ow, oh, w - ow, h - oh,
                             1.0, 1.0, GDK_INTERP_BILINEAR, 255);
    }
    
    xfce_scaled_image_set_from_pixbuf(XFCE_SCALED_IMAGE(smart->image), pb);
    g_object_unref(G_OBJECT(pb));
}

static void
smart_set_available(SmartPlugin *smart, gint packages)
{
  if (packages == 0)
  {
    char *s = _("No upgrades available");
    gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
    smart_set_icon(smart, smart->normal_icon, smart->updated_icon);
  }
  else if (packages > 0)
  {
    gchar *s = g_strdup_printf(_("%d upgrades available"), packages);
    gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
    smart_set_icon(smart, smart->normal_icon, smart->available_icon);
    g_free(s);
  }
  else
  {
    smart_set_icon(smart, smart->normal_icon, NULL);
  }
  smart->updates_available = packages;
}

static void
smart_update_channels_watch_func(GPid pid, gint status, gpointer user_data)
{
  SmartPlugin *smart = user_data;
  char buf[1024], *p;
  int packages;
  FILE *file;
  gchar *msg;
  gchar *s;
  int c;
  
  packages = -1;
  
  file = fdopen(smart->stdout, "r");
  if (file != NULL)
  {
     while ( (p = fgets(buf, sizeof(buf), file)) != NULL )
     {
       if (strstr(p, "Channels have ") != NULL)
       {
         p += strlen("Channels have ");
         if (p[0] == 'n' && p[1] == 'o')
           packages = 0;
         else
           packages = atoi(p);
       }
     }
    fclose(file);
  } 
 
  msg = NULL;
 
  file = fdopen(smart->stderr, "r");
  if (file != NULL)
  {
    while ( (p = fgets(buf, sizeof(buf), file)) != NULL )
    {
      if (strstr(p, "error: ") != NULL)
      {
        p += strlen("error: ");
        msg = g_strdup(p);
        p = msg + strlen(msg) - 1;
        if (*p == '\n') *p = '\0';
      }
    }  
    fclose(file);
  }
  
  if (status == 0 && packages >= 0)
  {
    /* restore icon */
    smart_set_available(smart, smart->updates_available);

    /* NOTE: "packages" is the number of new packages in the channels */
    if (packages == 0)
      s = g_strdup(_("Channels have no new packages."));
    else
      s = g_strdup_printf(_("Channels have %d new packages."), packages);

    gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
    g_free(s);
  }
  else
  {
    s = g_strdup_printf(_("Error updating%s%s"), msg ? ": " : "", msg ? msg : "");
    gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
    smart_set_icon(smart, smart->normal_icon, smart->error_icon);
    g_free(s);
  }

  if (msg)
  {
    if (strcmp(msg, "Configuration is in readonly mode.") == 0)
    {
        xfce_message_dialog(NULL, _("Smart Read-Only"), GTK_STOCK_DIALOG_WARNING,
                            _("Another process seems to be using Smart ?"), msg,
                            GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
    }
    g_free(msg);
  }
  
/*close(smart->stdout); closed above*/
/*close(smart->stderr); closed above*/
  g_spawn_close_pid(pid);

  smart->in_progress = FALSE;
}

static void
smart_update_channels_setup_child( gpointer   user_data)
{
  SmartPlugin *smart = user_data;
  /* nothing to do */
}

static int
smart_update_channels( SmartPlugin    *smart)
{
  gchar *argv[] = { SMART_UPDATE, NULL }; /* TODO: channels */
  gchar *envp[] = { "PATH=" SMART_PATH, "LANG=" SMART_LANG, NULL };
  GError *error = NULL;
  GPid pid;

  gtk_tooltips_set_tip(smart->tooltip, smart->button, _("Updating channels..."), NULL);
  smart_set_icon(smart, smart->normal_icon, smart->downloading_icon);
   
  smart->in_progress = TRUE;

  if (!g_spawn_async_with_pipes( NULL /*cwd*/, argv, envp,
    G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
    &smart_update_channels_setup_child, smart,
    &pid, NULL /*stdin*/, &smart->stdout, &smart->stderr, &error))
  {
    if (g_error_matches(error, G_SPAWN_ERROR, G_SPAWN_ERROR_NOENT))
    {
      /* smart-update not available */
      smart_sudo_smart_gui("update");
    }
    else
    {
      xfce_message_dialog(NULL, _("Smart Unavailable"), GTK_STOCK_DIALOG_ERROR,
                          _("Failed to run update"), error->message,
                          GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
    }

    g_error_free(error);
    smart->in_progress = FALSE;
    smart_set_available(smart, smart->updates_available);
    return 1;
  }
  else
  {
    g_child_watch_add(pid, smart_update_channels_watch_func, smart);
    return 0;
  }
 }

static gboolean
smart_timeout_function (gpointer user_data)
{
  SmartPlugin *smart = user_data;
  gint seconds;

  seconds = (gint) g_timer_elapsed(smart->timer, NULL);
  if (smart->in_progress || seconds < smart->check_interval * 60)
  {
    return TRUE;
  }
  
  smart_check_upgrades(smart);
  smart_set_interval(smart, smart->check_interval);
  
  return FALSE;
}

static void
smart_set_interval(SmartPlugin *smart, gint interval)
{
  if (smart->timeout)
    g_source_remove(smart->timeout);

  if (interval > 0)
  {
    g_timer_start(smart->timer);
    smart->timeout = g_timeout_add(TIMEOUT_TIME, smart_timeout_function, smart);
  }
  else
  {
    g_timer_stop(smart->timer);
    smart->timeout = 0;
  }

  smart->check_interval = interval;
}

static void
smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
{
  SmartPlugin *smart = user_data;
  char buf[1024], *p;
  int packages;
  FILE *file;
  gchar *msg;
  gchar *s;
  int c;
  
  packages = 0;
  
  file = fdopen(smart->stdout, "r");
  if (file != NULL)
  {
    while ( (p = fgets(buf, sizeof(buf), file)) != NULL )
    {
    }
    fclose(file);
  }

  msg = NULL;
  
  file = fdopen(smart->stderr, "r");
  if (file != NULL)
  {
    while ( (p = fgets(buf, sizeof(buf), file)) != NULL )
    {
      if (strstr(p, "error: ") != NULL)
      {
        p += strlen("error: ");
        msg = g_strdup(p);
        p = msg + strlen(msg) - 1;
        if (*p == '\n') *p = '\0';
      }
      else
      {
        /* TODO: read package names */
        if (*p != '\n') packages++;
      }
    }
    fclose(file);
  } 
 
  if (status == 0)
  {
    smart_set_available(smart, packages);
  }
  else
  {
    s = g_strdup_printf(_("Error checking%s%s"), msg ? ": " : "", msg ? msg : "");
    gtk_tooltips_set_tip(smart->tooltip, smart->button, s, NULL);
    smart_set_icon(smart, smart->normal_icon, smart->error_icon);
    g_free(s);
  }
 
  if (msg)
  {
    g_free(msg);
  }

/*close(smart->stdout); closed above */
/*close(smart->stderr); closed above */
  g_spawn_close_pid(pid);

  smart->in_progress = FALSE;
}

static void
smart_check_upgrades_setup_child( gpointer   user_data)
{
  SmartPlugin    *smart = user_data;
  /* nothing to do */
}

static int
smart_check_upgrades( SmartPlugin    *smart)
{
  gchar *argv[] = { SMART_PROGRAM, "upgrade", "--dump", NULL };
  gchar *envp[] = { "PATH=" SMART_PATH, "LANG=" SMART_LANG, NULL };
  GError *error = NULL;
  GPid pid;
  
  gtk_tooltips_set_tip(smart->tooltip, smart->button, _("Checking for upgrades..."), NULL);
  smart_set_icon(smart, smart->normal_icon, smart->downloading_icon);
  
  smart->in_progress = TRUE;
  
  if (!g_spawn_async_with_pipes( NULL /*cwd*/, argv, envp,
    G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_LEAVE_DESCRIPTORS_OPEN,
    &smart_check_upgrades_setup_child, smart,
    &pid, NULL /*stdin*/, &smart->stdout, &smart->stderr, &error))
  {
        xfce_message_dialog(NULL, _("Smart Unavailable"), GTK_STOCK_DIALOG_ERROR,
                            _("Failed to run upgrade"), error->message,
                            GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
        g_error_free(error);
        smart->in_progress = FALSE;
        smart_set_available(smart, smart->updates_available);
        return 1;
  }
  else
  {
    g_child_watch_add(pid, smart_check_upgrades_watch_func, smart);
    return 0;
  }
}

static gboolean
smart_program_exists(gchar *program)
{
  gchar *path = "/usr/bin"; /* TODO: check PATH */
  gchar *bin;
  gboolean ok;
  
  bin = g_strdup_printf("%s/%s", path, program);
  ok = g_file_test(bin, G_FILE_TEST_EXISTS);
  g_free(bin);

  return ok;
}

static void
smart_sudo_smart_gui(gchar *args)
{
  GError *error = NULL;
  gchar *program;
  gchar *command;
  gboolean ok;
  
  if (smart_program_exists(SMART_SUIDBIN))
    program = g_strdup(SMART_SUIDBIN);
  else /* if (smart_program_exists(SMART_SUDOBIN)) */
    program = g_strdup_printf("%s -- %s", SMART_SUDOBIN, SMART_PROGRAM);
/*else
    program = g_strdup(SMART_PROGRAM);*/
 
  command = g_strdup_printf("%s %s%s%s", program, "--gui",
                 args ? " " : "", args ? args : "");
  if (!xfce_exec(command, FALSE, FALSE, &error))
  {
    xfce_message_dialog(NULL, _("Smart Failed"), GTK_STOCK_DIALOG_ERROR,
                        _("Failed to run smart program"), error->message,
                        GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
    g_error_free(error);
  }

  g_free(program);
  g_free(command);
}

void
smart_save (XfcePanelPlugin *plugin,
             SmartPlugin    *smart)
{
  XfceRc *rc;
  gchar  *file;

  /* get the config file location */
  file = xfce_panel_plugin_save_location (plugin, TRUE);

  if (G_UNLIKELY (file == NULL))
    {
       DBG ("Failed to open config file");
       return;
    }

  /* open the config file, read/write */
  rc = xfce_rc_simple_open (file, FALSE);
  g_free (file);

  if (G_LIKELY (rc != NULL))
    {
      /* save the settings */
      xfce_rc_write_int_entry  (rc, "interval", smart->check_interval);
      xfce_rc_write_bool_entry (rc, "upgrade", smart->upgrade_directly);

      /* close the rc file */
      xfce_rc_close (rc);
    }
}



static void
smart_read (SmartPlugin *smart)
{
  XfceRc      *rc;
  gchar       *file;
  const gchar *value;
  
  /* get the plugin config file location */
  file = xfce_panel_plugin_save_location (smart->plugin, TRUE);

  if (G_LIKELY (file != NULL))
    {
      /* open the config file, readonly */
      rc = xfce_rc_simple_open (file, TRUE);

      /* cleanup */
      g_free (file);

      if (G_LIKELY (rc != NULL))
        {
          /* read the settings */
          smart->check_interval = xfce_rc_read_int_entry (rc, "interval", 0);
          smart->upgrade_directly =  xfce_rc_read_bool_entry (rc, "upgrade", TRUE);

          /* cleanup */
          xfce_rc_close (rc);

          /* leave the function, everything went well */
          return;
        }
    }

  /* something went wrong, apply default values */
  DBG ("Applying default settings");

  smart->check_interval = 0;
  smart->upgrade_directly = TRUE;
}



static gboolean
smart_button_press_cb(GtkWidget *w,
                          GdkEventButton *evt,
                          gpointer user_data)
{
    if(evt->button == 2)
        gtk_button_pressed(GTK_BUTTON(w));
    
    return FALSE;
}

static gboolean
smart_button_release_cb(GtkWidget *w, GdkEventButton *evt,
        gpointer user_data)
{
  SmartPlugin *smart = user_data;
    
  if(evt->x >= w->allocation.x
     && evt->x < w->allocation.x + w->allocation.width
     && evt->y >= w->allocation.y
     && evt->y < w->allocation.y + w->allocation.height)
  {
    switch(evt->button)
    {
      case 1:  /* left */
        if (smart->upgrade_directly && smart->updates_available > 0)
          smart_sudo_smart_gui("upgrade");
        else
          smart_sudo_smart_gui(NULL);
        break;
      case 2:  /* middle */
        if (smart->in_progress)
        {
           xfce_message_dialog(NULL, _("Upgrade In Progress"), GTK_STOCK_DIALOG_WARNING,
                               _("An upgrade is already in progress, please wait."), NULL,
                               GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
        }
        else
        {
           smart_check_upgrades(smart);
        }
        break;
    }
  }
  
  if(evt->button == 2)
    gtk_button_released(GTK_BUTTON(w));
    
  return FALSE;
}

static void
smart_menuitem_activate_cb(GtkMenuItem *item,
        gpointer user_data)
{
  SmartPlugin *smart = user_data;

  smart_update_channels(smart);
}

static void
smart_checkbox_toggle_cb(GtkToggleButton *button,
        gpointer user_data)
{
  SmartPlugin *smart = user_data;

  smart->upgrade_directly = gtk_toggle_button_get_active(button);
}

static void
smart_spin_value_change_cb(GtkSpinButton *button,
        gpointer user_data)
{
  SmartPlugin *smart = user_data;
  gint interval;
    
  interval = gtk_spin_button_get_value_as_int(button);
    
  if (smart->check_interval == 0 && interval > 0)
  {
    /* check immediately when first enabled */
    smart_check_upgrades(smart);
  }

  smart_set_interval (smart, interval);
}

static SmartPlugin *
smart_new (XfcePanelPlugin *plugin)
{
  SmartPlugin   *smart;
  GtkOrientation  orientation;
  GtkWidget *dummy;
  GtkWidget *menuicon;
  
  /* allocate memory for the plugin structure */
  smart = panel_slice_new0 (SmartPlugin);

  /* pointer to plugin */
  smart->plugin = plugin;

  smart->in_progress = FALSE;
  smart->updates_available = -1;

  smart->timer = g_timer_new();
  smart->timeout = 0;
 
  /* read the user settings */
  smart_read (smart);

  /* get the current orientation */
  orientation = xfce_panel_plugin_get_orientation (plugin);

  dummy = gtk_invisible_new();
  gtk_widget_realize(dummy);
    
  smart->normal_icon = xfce_themed_icon_load("xfce4-smart", 48);

  smart->available_icon = xfce_themed_icon_load("xfce4-smart-available", 16);
  smart->downloading_icon = xfce_themed_icon_load("xfce4-smart-downloading", 16);
  smart->updated_icon = xfce_themed_icon_load("xfce4-smart-updated", 16);
  smart->error_icon = xfce_themed_icon_load("xfce4-smart-error", 16);

  /* create some panel widgets */
  smart->tooltip = gtk_tooltips_new();
    
  smart->button = gtk_button_new();
  gtk_button_set_relief(GTK_BUTTON (smart->button), GTK_RELIEF_NONE);
  gtk_widget_show(smart->button);
  gtk_container_add(GTK_CONTAINER (plugin), smart->button);
  g_signal_connect(smart->button, "button-press-event",
            G_CALLBACK(smart_button_press_cb), smart);
  g_signal_connect(smart->button, "button-release-event",
            G_CALLBACK(smart_button_release_cb), smart);

  smart->image = xfce_scaled_image_new();
  gtk_widget_show(smart->image);
  gtk_container_add(GTK_CONTAINER (smart->button), smart->image);

  smart->menuitem = gtk_menu_item_new_with_label(_("Update Channels"));
  gtk_widget_show (smart->menuitem);
  menuicon = gtk_image_new_from_stock(GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU);
  gtk_widget_show (menuicon);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(smart->menuitem), menuicon);

  gtk_widget_destroy(dummy);

  /* set default icon and check */
  smart_set_available(smart, smart->updates_available);
  if (smart->check_interval > 0)
    smart_check_upgrades(smart);
  smart_set_interval(smart, smart->check_interval);

  return smart;
}

static void
smart_free (XfcePanelPlugin *plugin,
             SmartPlugin    *smart)
{
  GtkWidget *dialog;

  /* check if the dialog is still open. if so, destroy it */
  dialog = g_object_get_data (G_OBJECT (plugin), "dialog");
  if (G_UNLIKELY (dialog != NULL))
    gtk_widget_destroy (dialog);

  /* destroy the panel widgets */
  gtk_widget_destroy (smart->button);
  gtk_widget_destroy (smart->menuitem);

  gtk_object_sink(GTK_OBJECT (smart->tooltip));

  /* free the icons */
  g_object_unref(G_OBJECT(smart->normal_icon));
  g_object_unref(G_OBJECT(smart->available_icon));
  g_object_unref(G_OBJECT(smart->downloading_icon));
  g_object_unref(G_OBJECT(smart->updated_icon));
  g_object_unref(G_OBJECT(smart->error_icon));

  if(smart->timeout)
    g_source_remove(smart->timeout);
  if (G_LIKELY (smart->timer != NULL))
    g_timer_destroy (smart->timer);

  /* free the plugin structure */
  panel_slice_free (SmartPlugin, smart);
}

static void
smart_orientation_changed (XfcePanelPlugin *plugin,
                            GtkOrientation   orientation,
                            SmartPlugin    *smart)
{
}

static gboolean
smart_size_changed (XfcePanelPlugin *plugin,
                     gint             size,
                     SmartPlugin    *smart)
{
  GtkOrientation orientation;
  gint iconsize;

  /* get the orientation of the plugin */
  orientation = xfce_panel_plugin_get_orientation (plugin);

  /* set the widget size */
  if (orientation == GTK_ORIENTATION_HORIZONTAL)
    gtk_widget_set_size_request (GTK_WIDGET (plugin), -1, size);
  else
    gtk_widget_set_size_request (GTK_WIDGET (plugin), size, -1);

  iconsize = size - MAX(GTK_WIDGET(smart->button)->style->xthickness, 
                        GTK_WIDGET(smart->button)->style->ythickness) - 1;

  if (smart->normal_icon)
     g_object_unref(G_OBJECT(smart->normal_icon));

  smart->normal_icon = xfce_themed_icon_load("xfce4-smart", iconsize);

  gtk_widget_set_size_request(smart->button, size, size);
  smart_set_available(smart, smart->updates_available);

  /* we handled the orientation */
  return TRUE;
}


static void
smart_construct (XfcePanelPlugin *plugin)
{
  SmartPlugin *smart;

  /* setup transation domain */
  xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

  /* create the plugin */
  smart = smart_new (plugin);

  /* add the button to the panel */
  gtk_container_add (GTK_CONTAINER (plugin), smart->button);

  /* show the panel's right-click menu on this button */
  xfce_panel_plugin_add_action_widget (plugin, smart->button);

  /* connect plugin signals */
  g_signal_connect (G_OBJECT (plugin), "free-data",
                    G_CALLBACK (smart_free), smart);

  g_signal_connect (G_OBJECT (plugin), "save",
                    G_CALLBACK (smart_save), smart);

  g_signal_connect (G_OBJECT (plugin), "size-changed",
                    G_CALLBACK (smart_size_changed), smart);

  g_signal_connect (G_OBJECT (plugin), "orientation-changed",
                    G_CALLBACK (smart_orientation_changed), smart);

  /* show the configure menu item and connect signal */
  xfce_panel_plugin_menu_show_configure (plugin);
  g_signal_connect (G_OBJECT (plugin), "configure-plugin",
                    G_CALLBACK (smart_configure), smart);

  /* show the about menu item and connect signal */
  xfce_panel_plugin_menu_show_about (plugin);
  g_signal_connect (G_OBJECT (plugin), "about",
                    G_CALLBACK (smart_about), plugin);

  /* show the update menu item and connect signal */
  xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM(smart->menuitem));
  g_signal_connect (G_OBJECT (smart->menuitem), "activate",
                    G_CALLBACK (smart_menuitem_activate_cb), smart);
}


static void
smart_configure_response (GtkWidget    *dialog,
                           gint          response,
                           SmartPlugin *smart)
{
  gboolean result;

  if (response == GTK_RESPONSE_HELP)
    {
      /* show help */
      result = g_spawn_command_line_async ("exo-open --launch WebBrowser " SMART_WEBSITE, NULL);

      if (G_UNLIKELY (result == FALSE))
        g_warning (_("Unable to open the following url: %s"), SMART_WEBSITE);
    }
  else if (response == GTK_RESPONSE_ACCEPT)
    {
      if (smart->in_progress)
      {
        xfce_message_dialog(NULL, _("Update In Progress"), GTK_STOCK_DIALOG_WARNING,
                            _("An update is already in progress, please wait."), NULL,
                            GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
      }
      else
      {
        smart_sudo_smart_gui("update"); /* TODO: channels */
      }
    }
  else if (response == GTK_RESPONSE_OK)
    {
      /* remove the dialog data from the plugin */
      g_object_set_data (G_OBJECT (smart->plugin), "dialog", NULL);
      
      /* unlock the panel menu */
      xfce_panel_plugin_unblock_menu (smart->plugin);

      /* save the plugin */
      smart_save (smart->plugin, smart);

      /* destroy the properties dialog */
      gtk_widget_destroy (dialog);
    }
}

static GtkWidget *
smart_custom_button_new(const gchar *text, const gchar *icon)
{
    GtkWidget *btn, *hbox, *img, *lbl;
    GdkPixbuf *pix;
    gint iw, ih;
    
    g_return_val_if_fail((text && *text) || icon, NULL);
    
    btn = gtk_button_new();
    
    hbox = gtk_hbox_new(FALSE, 4);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
    gtk_widget_show(hbox);
    gtk_container_add(GTK_CONTAINER(btn), hbox);
    
    if(icon) {
        img = gtk_image_new_from_stock(icon, GTK_ICON_SIZE_BUTTON);
        if(!img || gtk_image_get_storage_type(GTK_IMAGE(img)) == GTK_IMAGE_EMPTY) {
            gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &iw, &ih);
            pix = xfce_themed_icon_load(icon, iw);
            if(pix) {
                if(img)
                    gtk_image_set_from_pixbuf(GTK_IMAGE(img), pix);
                else
                    img = gtk_image_new_from_pixbuf(pix);
                g_object_unref(G_OBJECT(pix));
            }
        }
        if(img) {
            gtk_widget_show(img);
            gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
        }
    }
    
    if(text) {
        lbl = gtk_label_new_with_mnemonic(text);
        gtk_widget_show(lbl);
        gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
        gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), btn);
    }
    
    return btn;
}

void
smart_configure (XfcePanelPlugin *plugin,
                  SmartPlugin    *smart)
{
  GtkWidget *dialog;
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *vbox, *hbox;
   
  /* block the plugin menu */
  xfce_panel_plugin_block_menu (plugin);

  /* create the dialog */
  dialog = xfce_titled_dialog_new_with_buttons (_("Smart Plugin"),
                                                GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
                                                GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
                                              /*GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                                GTK_STOCK_CLOSE, GTK_RESPONSE_OK,*/
                                                NULL);

  button = gtk_button_new_from_stock(GTK_STOCK_HELP);
  gtk_widget_show(button);
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, GTK_RESPONSE_HELP);

  button = smart_custom_button_new(_("_Update..."), GTK_STOCK_REFRESH);
  gtk_widget_show(button);
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, GTK_RESPONSE_ACCEPT);

  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  gtk_widget_show(button);
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button, GTK_RESPONSE_OK);

  vbox=gtk_vbox_new (FALSE,0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);

  button = gtk_check_button_new_with_label(_("Upgrade on launch (if upgrades available)"));
  gtk_widget_show (button);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), smart->upgrade_directly);
  g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox_toggle_cb), smart);
  gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);

  hbox = gtk_hbox_new (FALSE,0);
  gtk_widget_show (hbox);
  label = gtk_label_new(_("Check for upgrades every "));
  gtk_widget_show (label);
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
  button=gtk_spin_button_new_with_range(0,60,1);
  gtk_widget_show (button);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(button), smart->check_interval);
  g_signal_connect(G_OBJECT(button),"value-changed", G_CALLBACK(smart_spin_value_change_cb), smart);
  gtk_box_pack_start (GTK_BOX(hbox),button,FALSE,FALSE,0);
  label = gtk_label_new(_(" minutes"));
  gtk_widget_show (label);
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

  /* center dialog on the screen */
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

  /* set dialog icon */
  gtk_window_set_icon_name (GTK_WINDOW (dialog), "xfce4-settings");

  /* link the dialog to the plugin, so we can destroy it when the plugin
   * is closed, but the dialog is still open */
  g_object_set_data (G_OBJECT (plugin), "dialog", dialog);

  /* connect the reponse signal to the dialog */
  g_signal_connect (G_OBJECT (dialog), "response",
                    G_CALLBACK(smart_configure_response), smart);

  /* show the entire dialog */
  gtk_widget_show (dialog);
}

void
smart_about (XfcePanelPlugin *plugin)
{
  XfceAboutInfo *info;
  GtkWidget *dialog;
  GdkPixbuf *icon;

  info = xfce_about_info_new(
              _("Smart Plugin"),
              VERSION,
              _("Check for updates using the Smart Package Manager"),
              XFCE_COPYRIGHT_TEXT("2007", "Anders F Bjorklund"),
              XFCE_LICENSE_GPL);
  xfce_about_info_set_homepage(info, SMART_WEBSITE);

  icon = xfce_themed_icon_load("xfce4-smart", 48);

  dialog = xfce_about_dialog_new(NULL, info, icon);
  gtk_window_set_default_size(GTK_WINDOW(dialog), 500, 400);

  xfce_about_info_free(info);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy (dialog);
  g_object_unref(G_OBJECT(icon));
}
